import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Tennessee raw data for consolidation."""

    postal_code = "TN"
    fields = dict(
        company="Company",
        location=lambda row: row["City"] or f"{row['County']} County",
        notice_date="Notice Date",
        effective_date="Effective Date",
        jobs="No. Of Employees",
    )
    date_format = (
        "%Y/%m/%d",
        "%m/%d/%Y",
        "%B %d, %Y",
        "%B %Y",
        "%B%d, %Y",
        "%B %d. %Y",
        "%B %d,%Y",
    )
    date_corrections = {
        "2018/4/ 27": datetime(2018, 4, 27),
        "start of layoff -\xa0March 13, 2020": datetime(2020, 3, 13),
        "December 15-30, 2020": datetime(2020, 12, 15),
        "June 17 - June 30, 2020": datetime(2020, 6, 17),
        "124": None,
        "March 16, 2020 - June 30, 2020": datetime(2020, 3, 16),
        "March 20-24, 2020": datetime(2020, 3, 20),
        "March 4, 2020 - June 4, 2020": datetime(2020, 3, 4),
        "Apri 1, 2020": datetime(2020, 4, 1),
        "Beginning April 14 and ending June 30, 2020": datetime(2020, 4, 14),
        "Beginning February 16 and ending February 29, 2020": datetime(2020, 2, 16),
        "Beginning March 17 and ending March 30, 2020": datetime(2020, 3, 17),
        "Beginning in February 2020": datetime(2020, 2, 1),
        "beginning February 2020": datetime(2020, 2, 1),
        "January 3, 2020 through January 31, 2020": datetime(2020, 1, 3),
        "December 13, 2019 and continue until February 28, 2020": datetime(
            2019, 12, 13
        ),
        "December 1, 2019 until December 31, 2019": datetime(2019, 12, 1),
        "Will begin on November 30, 2019 and will continue to December 31, 2020": datetime(
            2019, 11, 30
        ),
        "Will begin on October 7, 2019 and will continue to November 30, 2019": datetime(
            2019, 10, 7
        ),
        "Initial layoff September 4, 2019 with additional layoffs planned September 13 and September 20": datetime(
            2019, 9, 4
        ),
        "November 9 through November 23, 2019": datetime(2019, 11, 23),
        "September 30, 2019 for 4 employees and October 31, 2019 for 174 employees": datetime(
            2019, 9, 30
        ),
        "Late September 2019": datetime(2019, 9, 15),
        "September 20, 2019 and continuing through December 2019": datetime(
            2019, 9, 20
        ),
        "August 30, 2019 through December 31, 2019": datetime(2019, 8, 30),
        "April 22, 2019, May 4, 2019, and August 7, 2019": datetime(2019, 4, 22),
        "March 3, 2019, March 11, 2019,": datetime(2019, 3, 3),
        "June 15, 2018, July 6, 2018, August 3, 2018": datetime(2018, 6, 15),
        "July 31, 2023; September 30, 2023; December 31, 2023": datetime(2023, 7, 31),
        "June 12, 2023\xa0– August 11, 2023": datetime(2023, 6, 12),
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        try:
            return super().transform_date(value)
        except Exception:
            value = value.strip().split(" and ")[0].strip()
            value = value.strip().split(" to ")[0].strip()
            value = value.strip().split(" through ")[0].strip()
            value = value.strip().split(" & ")[0].strip()
            value = value.strip().split(" – ")[0].strip()
            return super().transform_date(value)



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Oklahoma raw data for consolidation."""

    postal_code = "OK"
    fields = dict(
        company="employer",
        location=lambda row: row["city"] or row["address"] or row["lwib_area"],
        notice_date="notice_date",
        jobs="number_of_employees_affected",
    )
    date_format = "%b %d, %Y"



import logging
import typing
from pathlib import Path

import click

from . import consolidate as consolidate_runner
from . import download as download_runner
from . import integrate as integrate_runner
from . import utils


@click.group()
def cli():
    """Consolidate, enrich and republish the data gathered by warn-scraper."""
    pass


@cli.command()
@click.option(
    "--download-dir",
    default=utils.WARN_TRANSFORMER_OUTPUT_DIR / "raw",
    type=click.Path(),
    help="The Path were the results will be downloaded",
)
@click.option(
    "--source",
    default=None,
    help="The source to download. Default is all sources.",
)
@click.option(
    "--log-level",
    "-l",
    default="INFO",
    type=click.Choice(
        ("DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"), case_sensitive=False
    ),
    help="Set the logging level",
)
def download(
    download_dir: Path, source: typing.Optional[str] = None, log_level: str = "INFO"
):
    """Download all the CSVs in the WARN Notice project on biglocalnews.org."""
    logging.getLogger("urllib3").setLevel(logging.ERROR)
    logging.basicConfig(level=log_level, format="%(asctime)s - %(name)s - %(message)s")
    logger = logging.getLogger(__name__)
    logger.debug("Running download command")
    download_runner.run(download_dir, source)


@cli.command()
@click.option(
    "--input-dir",
    default=utils.WARN_TRANSFORMER_OUTPUT_DIR / "raw",
    type=click.Path(),
    help="The Path were the raw files results are located",
)
@click.option(
    "--source",
    default=None,
    help="The source to download. Default is all sources.",
)
@click.option(
    "--log-level",
    "-l",
    default="INFO",
    type=click.Choice(
        ("DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"), case_sensitive=False
    ),
    help="Set the logging level",
)
def consolidate(
    input_dir: Path, source: typing.Optional[str] = None, log_level: str = "INFO"
):
    """Consolidate raw data using a common data schema."""
    logging.basicConfig(level=log_level, format="%(asctime)s - %(name)s - %(message)s")
    logger = logging.getLogger(__name__)
    logger.debug("Running consolidate command")
    consolidate_runner.run(input_dir, source)


@cli.command()
@click.option(
    "--input-dir",
    default=utils.WARN_TRANSFORMER_OUTPUT_DIR / "processed" / "consolidated.csv",
    type=click.Path(),
    help="The Path were the new results are located",
)
@click.option(
    "--init",
    default=False,
    is_flag=True,
)
@click.option(
    "--log-level",
    "-l",
    default="INFO",
    type=click.Choice(
        ("DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"), case_sensitive=False
    ),
    help="Set the logging level",
)
def integrate(input_dir: Path, init: bool = False, log_level: str = "INFO"):
    """Integrate the latest consolidated data with the current database."""
    logging.getLogger("urllib3").setLevel(logging.ERROR)
    logging.basicConfig(level=log_level, format="%(asctime)s - %(name)s - %(message)s")
    logger = logging.getLogger(__name__)
    logger.debug("Running integrate command")
    integrate_runner.run(input_dir, init_current_data=init)


if __name__ == "__main__":
    cli()



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Missouri raw data for consolidation."""

    postal_code = "MO"
    fields = dict(
        company="Title",
        location="Location(s)",
        notice_date="Received Sort descending",
        effective_date="Layoff date(s)",
        jobs="# affected",
    )
    date_format = ("%m/%d/%Y", "%m/%d/%y", "%B %Y", "%B %d, %Y")
    date_corrections = {
        "04/-9/2020": datetime(2020, 4, 9),
        "March 2020": datetime(2020, 3, 1),
        "": None,
        "11/08/2109": datetime(2019, 11, 8),
    }
    jobs_corrections = {
        "330 remote workers (18 located in Missouri)": 18,
        "Unknown": None,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        try:
            return super().transform_date(value)
        except Exception:
            value = value.strip().split()[0].strip()
            value = value.strip().split("-")[0].strip()
            value = value.replace("–", "")
            value = value.replace(",", "")
            return super().transform_date(value)

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "clos" in row["Type"].lower() or None



import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Virginia raw data for consolidation."""

    postal_code = "VA"
    fields = dict(
        company="Company Name",
        location="Location City",
        notice_date="Notice Date",
        effective_date="Impact Date",
        jobs="Employees Affected",
    )
    date_format = "%m/%d/%Y"
    date_corrections = {
        "10/01/1973": None,
    }

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "yes" in row["Closure"].lower() or None



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform South Dakota raw data for consolidation."""

    postal_code = "SD"
    fields = dict(
        company="Company",
        location="Location",
        notice_date="Date Received",
        jobs="Employees Affected",
    )
    date_format = "%m/%d/%Y"
    jobs_corrections = {
        "1-5": 1,
    }



import tempfile
from pathlib import Path

import pytest

from warn_transformer import download


@pytest.mark.runvcr
@pytest.mark.vcr()
def test_download():
    """Test download."""
    download.run(Path(tempfile.gettempdir()))



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Rhode Island raw data for consolidation."""

    postal_code = "RI"
    fields = dict(
        company="Company Name (* Denotes Covid 19 Related WARN)",
        location="Location of Layoffs",
        date="WARN Date",
        effective_date="Effective Date",
        jobs="Number Affected",
    )
    date_format = ("%Y-%m-%d %H:%M:%S", "%m/%d/%Y", "%m/%d/%y", "%m/%Y")
    date_corrections = {
        "2108-10-23 00:00:00": datetime(2018, 10, 23),
        "2108-11-01 00:00:00": datetime(2018, 11, 1),
        "Staggered": None,
    }
    jobs_corrections = {
        "---": None,
        "54 Union 3 Non Union": 57,
        "190 company with an additional 100 contracted": 290,
        "60-80": 60,
        "additional 16": 16,
        "1900-03-17 00:00:00": None,
        "309 *updated 10/26/23": 309,
        "1900-01-01 00:00:00": 1,
    }

    def transform_company(self, value: str) -> str:
        """Transform a raw company name.

        Args:
            value (str): The raw company string provided by the source

        Returns: A string object ready for consolidation.
        """
        return value.strip().replace("*", "")

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        try:
            return super().transform_date(value)
        except Exception:
            value = value.strip().split()[0].strip()
            value = value.strip().split("-")[0].strip()
            value = value.replace("–", "")
            value = value.replace(",", "")
            return super().transform_date(value)

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "yes" in row["Closing Yes/No"].lower() or None



import csv
import hashlib
import json
import logging
import typing
from datetime import datetime, timedelta
from pathlib import Path

from marshmallow import Schema, fields

logger = logging.getLogger(__name__)


class WarnNoticeSchema(Schema):
    """An standardized instance of a WARN Act Notice."""

    hash_id = fields.Str(required=True)
    postal_code = fields.Str(max_length=2, required=True)
    company = fields.Str(required=True)
    location = fields.Str(required=True, allow_none=True)
    notice_date = fields.Date(required=True, allow_none=True)
    effective_date = fields.Date(required=True, allow_none=True)
    jobs = fields.Int(required=True, allow_none=True)
    is_temporary = fields.Boolean(required=True, allow_none=True, default=None)
    is_closure = fields.Boolean(required=True, allow_none=True, default=None)
    is_amendment = fields.Boolean(required=True, default=False)


class BaseTransformer:
    """Transform a state's raw data for consolidation."""

    schema = WarnNoticeSchema

    # The base attributes that need to be defined on all subclasses.
    postal_code: str = "xx"
    fields: typing.Dict = dict()

    # The default date format. It will need to be customized by source.
    date_format: typing.Any = "%m/%d/%Y"
    # Manual date corrections for malformed data
    date_corrections: typing.Dict = {}
    # How many days in the future are allowed
    max_future_days: int = 365
    # The minimum year allowed
    minimum_year: int = 1988

    # Manual jobs corrections for malformed data
    jobs_corrections: typing.Dict = {}
    # The max jobs we allow without throwing an error
    maximum_jobs: int = 10000

    def __init__(self, input_dir: Path):
        """Intialize a new instance.

        Args:
            input_dir (Path): A directory where our raw data is stored
        """
        self.input_dir = input_dir
        self.raw_data = self.get_raw_data()

    def get_raw_data(self) -> typing.List[typing.Dict]:
        """Get the raw data from our scraper for this source.

        Returns: A list of raw rows of data from the source.
        """
        # Get downloaded file
        raw_path = self.input_dir / f"{self.postal_code.lower()}.csv"
        # Open the csv
        with open(raw_path) as fh:
            reader = csv.DictReader(fh)
            # Return it as a list
            return list(reader)

    def transform(self) -> typing.List[typing.Dict]:
        """Transform prepared rows into a form that's ready for consolidation.

        Returns: A validated list of dictionaries that conform to our schema
        """
        logger.debug(f"Transforming {self.postal_code}")

        # Prep the row list for transformation
        row_list = self.prep_row_list(self.raw_data)

        # Transform the row list into dicts that are ready to be submitted for validation
        transformed_list = [self.transform_row(r) for r in row_list]

        # Validate each row against our schema
        validated_list = [self.schema().load(r) for r in transformed_list]

        # Deal with amendments
        amended_list = self.handle_amendments(validated_list)

        # Return the result, which should be ready for consolidation
        return amended_list

    def prep_row_list(
        self, row_list: typing.List[typing.Dict]
    ) -> typing.List[typing.Dict]:
        """Make necessary transformations to the raw row list prior to transformation.

        Args:
            row_list (list): A list of raw rows of data from the source.

        Returns: The row list minus empty records
        """
        prepped_list = []
        for row in row_list:
            # Skip empty rows
            try:
                # A list with only empty cells will throw an error
                next(v for v in row.values() if v.strip())
            except StopIteration:
                continue
            prepped_list.append(row)
        return prepped_list

    def transform_row(self, row: typing.Dict) -> typing.Dict:
        """Transform a row into a form that's ready for consolidation.

        Args:
            row (dict): One raw row of data from the source

        Returns: A transformed dict that's ready to be loaded into our consolidated schema.
        """
        # Parse the fields we expect in every transformer
        data = dict(
            postal_code=self.postal_code.upper(),
            company=self.transform_company(
                self.get_raw_value(row, self.fields["company"])
            ),
            location=self.transform_location(
                self.get_raw_value(row, self.fields["location"])
            ),
            jobs=self.transform_jobs(self.get_raw_value(row, self.fields["jobs"])),
            is_temporary=self.check_if_temporary(row),
            is_closure=self.check_if_closure(row),
            is_amendment=self.check_if_amendment(row),
        )

        # Add optional date fields
        if "notice_date" in self.fields:
            data["notice_date"] = self.transform_date(
                self.get_raw_value(row, self.fields["notice_date"])
            )
        else:
            data["notice_date"] = None

        if "effective_date" in self.fields:
            data["effective_date"] = self.transform_date(
                self.get_raw_value(row, self.fields["effective_date"])
            )
        else:
            data["effective_date"] = None

        # Stamp record with a unique ID
        data["hash_id"] = self.get_hash_id(data)

        # Return the results
        return data

    def get_raw_value(self, row, method):
        """Fetch a value from the row that for transformation.

        Args:
            row: One raw row of data from the source
            method: The technique to use to pull data.
                If a strong method is provided,
                it is used to fetch a key of that name from the row. If a callable function is provided, the row is run through it.

        Returns: A value ready for transformation.
        """
        # If a string is provided, pull it from the row dict.
        if isinstance(method, str):
            return row[method]
        # If a function is provided, run the row through it.
        elif isinstance(method, typing.Callable):
            return method(row)
        else:
            raise ValueError("The field method your provided is not valid.")

    def get_hash_id(self, data: typing.Dict) -> str:
        """Convert the row into a unique hexdigest to use as a unique identifier.

        Args:
            data (dict): One raw row of data from the source

        Returns: A unique hexdigest string computed from the source data.
        """
        dict_string = json.dumps(data)
        hash_obj = hashlib.sha224(dict_string.encode("utf-8"))
        return hash_obj.hexdigest()

    def transform_company(self, value: str) -> str:
        """Transform a raw company name.

        Args:
            value (str): The raw company string provided by the source

        Returns: A string object ready for consolidation.
        """
        return value.strip()

    def transform_location(self, value: str) -> str:
        """Transform a raw location.

        Args:
            value (str): The raw location string provided by the source

        Returns: A string object ready for consolidation.
        """
        return value.strip()

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        # Clean up the string
        value = value.strip()

        # If there's nothing to convert, return None
        if not value:
            return None

        # The result, whene we find it
        dt: typing.Any = None

        # If there's only one date_format, try that
        if isinstance(self.date_format, str):
            try:
                dt = datetime.strptime(value, self.date_format)
            except ValueError:
                logger.debug(f"Could not parse '{value}'. Looking up correction")
                dt = self.date_corrections[value]

        # If it's a list, try them one by one
        elif isinstance(self.date_format, (list, tuple)):
            for f in self.date_format:
                try:
                    dt = datetime.strptime(value, f)
                except ValueError:
                    continue
            # If there's nothing at the end of the loop, try the correction
            if not dt:
                logger.debug(f"Could not parse '{value}'. Looking up correction")
                dt = self.date_corrections[value]

        # If the date parses as None, return that
        if dt is None:
            return None

        # Make sure we've got a date at this point
        assert dt is not None and isinstance(dt, datetime)

        # If the date is more than 365 days in future, fix it
        today = datetime.today()
        if dt > today + timedelta(days=self.max_future_days):
            logger.debug(
                f"Date '{dt}' is more than {self.max_future_days} days in the future. Looking up correction"
            )
            dt = self.date_corrections[value]

        # If the date is below the minimum year, fix it
        if dt.year < self.minimum_year:
            logger.debug(
                f"Year {dt.year} below minimum of {self.minimum_year}. Looking up correction"
            )
            dt = self.date_corrections[value]

        # If the date parses as None, return that
        if dt is None:
            return None

        # Make sure we've got a date at this point
        assert dt is not None and isinstance(dt, datetime)

        # If we have a datetime, return the result as a string
        return str(dt.date())

    def transform_jobs(self, value: str) -> typing.Optional[int]:
        """Transform a raw jobs number into an integer.

        Args:
            value (str): A raw jobs number provided by the source

        Returns: An integer number ready for consolidation. Or, if the value is invalid, a None.
        """
        # If there's nothing there, return None
        if not value:
            return None

        # Cut whitespace
        value = value.strip()

        # If there's nothing there, return None
        if not value:
            return None

        # Cut any commas
        value = value.replace(",", "")
        try:
            # Convert to integer
            clean_value = int(float(value))
        except ValueError:
            # If it won't convert, look for a manual correction
            logger.debug(f"Could not parse '{value}'. Looking up correction")
            clean_value = self.jobs_corrections[value]

        # If it's None, return it now
        if not clean_value:
            return clean_value

        # Now validate it
        if clean_value < 0:
            logger.debug("Jobs must be greater than 0. Looking up correction")
            clean_value = self.jobs_corrections[clean_value]
        if clean_value > self.maximum_jobs:
            logger.debug(
                f"Jobs greater than {self.maximum_jobs} are probably wrong. Looking up correction"
            )
            clean_value = self.jobs_corrections[clean_value]

        # Pass it out
        return clean_value

    def check_if_temporary(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a temporary or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return None

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return None

    def check_if_amendment(self, row: typing.Dict) -> bool:
        """Determine whether a row is an amendment or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean
        """
        return False

    def handle_amendments(
        self, row_list: typing.List[typing.Dict]
    ) -> typing.List[typing.Dict]:
        """Remove amended filings from the provided list of records.

        Args:
            row_list (list): A list of clean rows of data.

        Returns: A list of cleaned data, minus amended records.
        """
        amendments_count = len([r for r in row_list if r["is_amendment"] is True])
        if amendments_count == 0:
            logger.debug(f"No amendments in {self.postal_code}")
            return row_list
        raise NotImplementedError



import csv
import json
import logging
import typing
from collections import defaultdict
from datetime import datetime, timezone
from itertools import chain
from operator import itemgetter
from pathlib import Path

import jellyfish
import requests

from . import utils

logger = logging.getLogger(__name__)


def run(
    new_path: Path = utils.WARN_TRANSFORMER_OUTPUT_DIR
    / "processed"
    / "consolidated.csv",
    init_current_data: bool = False,
) -> Path:
    """Integrate new consolidated data with the current database.

    Args:
        new_path (Path): The path to the latest consolidated file on the local file system
        init_current_data (bool): Set to True when you want to create a new integrated dataset from scratch. Default False.

    Returns a Path to the newly integrated file.
    """
    # Get the most recently published integrated dataset
    current_data_list = get_current_data(init_current_data)

    # Read in new consolidated.csv file
    with open(new_path) as fh:
        new_data_reader = csv.DictReader(fh)
        new_data_list = list(new_data_reader)
    logger.debug(f"{len(new_data_list)} records in new file at {new_path}")

    # Regroup each list by state
    current_data_by_source = regroup_by_source(current_data_list)
    new_data_by_source = regroup_by_source(new_data_list)

    # Winnow down the new data to records that have changed
    changed_data_by_source = get_changed_data(
        new_data_by_source, current_data_by_source
    )

    # Loop through the changed data to determine which are new and which are amendements
    amend_by_source = {}
    insert_by_source = {}
    for postal_code, change_list in changed_data_by_source.items():
        logger.debug(
            f"Inspecting {len(change_list)} changed records from {postal_code}"
        )
        current_row_list = current_data_by_source[postal_code]
        amend_list = []
        insert_list = []
        for new_row in change_list:
            # See if we can find a likely parent that was amended
            likely_ancestor = get_likely_ancestor(new_row, current_row_list)
            # If there is one, we assume this is an amendment
            if likely_ancestor:
                amend_list.append({"new": new_row, "current": likely_ancestor})
            # Otherwise we estimate it's a new record
            else:
                insert_list.append(new_row)

        # Log the result
        logger.debug(f"{len(insert_list)} new records")
        logger.debug(f"{len(amend_list)} amended records")

        # Add to master list for integration
        insert_by_source[postal_code] = insert_list
        amend_by_source[postal_code] = amend_list

    # Final report on what we'll do
    full_amend_list = flatten_grouped_data(amend_by_source)
    full_insert_list = flatten_grouped_data(insert_by_source)
    logger.debug(f"{len(full_insert_list)} total new records")
    logger.debug(f"{len(full_amend_list)} total amended records")

    # Write out both lists
    insert_path = utils.WARN_TRANSFORMER_OUTPUT_DIR / "processed" / "additions.csv"
    with open(insert_path, "w") as fh:
        if full_insert_list:
            logger.debug(f"Writing {len(full_insert_list)} records to {insert_path}")
            writer = csv.DictWriter(fh, full_insert_list[0].keys())
            writer.writeheader()
            writer.writerows(full_insert_list)

    amend_path = utils.WARN_TRANSFORMER_OUTPUT_DIR / "processed" / "amendments.csv"
    with open(amend_path, "w") as fh:
        if full_amend_list:
            logger.debug(f"Writing {len(full_amend_list)} records to {amend_path}")
            writer = csv.DictWriter(fh, full_amend_list[0].keys())
            writer.writeheader()
            writer.writerows(full_amend_list)

    # Create a new list to store everything
    integrated_list: typing.List[typing.Dict[str, typing.Any]] = []

    # Create a lookup of the current amended records that links them their likely replacements
    amend_lookup = {d["current"]["hash_id"]: d["new"] for d in full_amend_list}

    # Get the current timestamp to mark the updates we make in this run
    now = datetime.now(timezone.utc)

    # Loop through everything in the current database
    for current_row in current_data_list:
        # If this is an amended row ...
        if current_row["hash_id"] in amend_lookup:
            # Pull out the new record from the our lookup
            amended_row = amend_lookup[current_row["hash_id"]]

            # Link it to its likely ancestor
            amended_row["is_amendment"] = True
            amended_row["is_superseded"] = False
            amended_row["likely_ancestor"] = current_row["hash_id"]

            # Update is metadata
            amended_row["first_inserted_date"] = current_row["first_inserted_date"]
            amended_row["last_updated_date"] = now
            amended_row["estimated_amendments"] = (
                current_row["estimated_amendments"] + 1
            )

            # Add it to the new integrated database
            integrated_list.append(amended_row)

            # Mark the current record as superseded
            # This allows it to be excluded circumstances where we only want unique records
            # But without deleting it entirely
            current_row["is_superseded"] = True

            # Add it to the integrated database
            integrated_list.append(current_row)

        # If the current row is not amended ...
        else:
            # If these field haven't already been filled in, nope 'em
            if "is_superseded" not in current_row.keys():
                current_row["is_superseded"] = False
            if "is_amendment" not in current_row.keys():
                current_row["is_amendment"] = False
            # Then we just keep what we got
            integrated_list.append(current_row)

    # Now insert the new records with today's timestamp
    for new_row in full_insert_list:
        new_row["first_inserted_date"] = now
        new_row["last_updated_date"] = now
        new_row["estimated_amendments"] = 0
        new_row["is_superseded"] = False
        new_row["is_amendment"] = False
        integrated_list.append(new_row)

    # And sort everything in reverse chronological order
    sorted_list = sorted(
        integrated_list,
        key=itemgetter("last_updated_date", "first_inserted_date", "notice_date"),
        reverse=True,
    )

    # Finally, write out what we got
    integrated_path = utils.WARN_TRANSFORMER_OUTPUT_DIR / "processed" / "integrated.csv"
    logger.debug(f"Writing {len(integrated_list)} records to {integrated_path}")
    with open(integrated_path, "w") as fh:
        fieldnames = [
            "hash_id",
            "first_inserted_date",
            "notice_date",
            "effective_date",
            "postal_code",
            "company",
            "location",
            "jobs",
            "is_closure",
            "is_temporary",
            "is_superseded",
            "is_amendment",
            "likely_ancestor",
            "estimated_amendments",
            "last_updated_date",
        ]
        writer = csv.DictWriter(fh, fieldnames, extrasaction="ignore")
        writer.writeheader()
        writer.writerows(sorted_list)

    # And return the path
    return integrated_path


def is_similar_string(s1, s2):
    """Evaluate whether we consider the two strings close enough to be likely variations.

    Args:
        s1 (str): The first string.
        s2 (str): The second string.

    Returns True or False.
    """
    return jellyfish.jaro_winkler_similarity(s1, s2) > 0.95


def is_similar_date(d1, d2):
    """Evaluate whether we consider the two date strings close enough to be likely variations.

    Args:
        d1 (str): The first string.
        d2 (str): The second string.

    Returns True or False.
    """
    return jellyfish.levenshtein_distance(d1, d2) <= 3


def get_likely_ancestor(
    new_row: typing.Dict[str, typing.Any], current_data: typing.List
) -> typing.Optional[typing.Dict[str, typing.Any]]:
    """Determine if the provided new row has a likely parent in the current dataset.

    Args:
        new_row (dict): A record from the new dataset believed to contain a change to the current dataset.
        current_data (list): All of the records in the current dataset for comparison

    Returns:
        The record in the current data judged most likely to be the ancestor of the new record.
        Returns None if the record is estimated to be new.
    """
    # Check our key fields against everything in the dataset
    likely_match_list = []
    for current_row in current_data:
        # Check the company names
        if not is_similar_string(new_row["company"], current_row["company"]):
            continue

        # Check the notice date
        if not is_similar_date(new_row["notice_date"], current_row["notice_date"]):
            continue

        # Check the effective date, if it exists
        if new_row["effective_date"] and current_row["effective_date"]:
            if not is_similar_date(
                new_row["effective_date"], current_row["effective_date"]
            ):
                continue

        # Check the location, if it exists
        if new_row["location"] and current_row["location"]:
            if not is_similar_string(new_row["location"], current_row["location"]):
                continue

        # Whatever is left we keep
        likely_match_list.append(current_row)

    # If there's nothing, return None
    if not likely_match_list:
        return None

    # If there is more than one likely match, we are going to log out
    if len(likely_match_list) > 1:
        # Log here. Might do more later.
        logger.debug("New row has more than one likely match")
        logger.debug(f"New row: {json.dumps(new_row, indent=2, default=str)}")
        logger.debug(
            f"Likely matches: {json.dumps(likely_match_list, indent=2, default=str)}"
        )

    # For now we just return the first one
    return likely_match_list[0]


def get_current_data(init: bool = False) -> typing.List[typing.Dict[str, typing.Any]]:
    """Fetch the most recent published version of our integrated dataset.

    Args:
        init (bool): Set to True when you want to create a new integrated dataset from scratch. Default False.

    Returns a list of dictionaries ready for comparison against the new consolidated data file.
    """
    # Set which file to pull
    base_url = "https://raw.githubusercontent.com/biglocalnews/warn-github-flow/transformer/data/warn-transformer/processed/"
    if init:
        current_url = f"{base_url}consolidated.csv"
        logger.debug(f"Initializing new current file from {current_url}")
    else:
        current_url = f"{base_url}integrated.csv"
        logger.debug(f"Downloading most recent current file from {current_url}")

    # Download the current database
    current_r = requests.get(current_url)
    current_data_str = current_r.content.decode("utf-8")

    # Read in the current database
    current_data_reader = csv.DictReader(current_data_str.splitlines(), delimiter=",")
    current_data_list: typing.List[typing.Dict[str, typing.Any]] = list(
        current_data_reader
    )

    # Get the current timestamp to mark the updates we make in this run
    now = datetime.now(timezone.utc)

    # If we're initializing a new dataset, we'll need to fill in the extra
    # fields custom to the integrated set.
    if init:
        for row in current_data_list:
            row["first_inserted_date"] = now
            row["last_updated_date"] = now
            row["estimated_amendments"] = 0
    else:
        for row in current_data_list:
            # Otherwise we'll want to parse a few data types for later use
            row["last_updated_date"] = datetime.fromisoformat(row["last_updated_date"])
            row["first_inserted_date"] = datetime.fromisoformat(
                row["first_inserted_date"]
            )
            row["estimated_amendments"] = int(row["estimated_amendments"])

    # Return the list
    logger.debug(f"{len(current_data_list)} records downloaded from current database")
    return current_data_list


def get_changed_data(
    new_data: typing.DefaultDict[str, typing.List],
    current_data: typing.DefaultDict[str, typing.List],
) -> typing.DefaultDict[str, typing.List]:
    """Determine which rows in a new data file are different from the current dataset.

    Args:
        new_data (dict): A dictionary keyed by postal code. Each value is a list of all records from that source.
        current_data (dict): A dictionary keyed by postal code. Each value is a list of all records from that source.

    Returns a dictionary keyed by postal code. Each value is a list of all records with that value deemed to have changed.
    """
    changed_dict = defaultdict(list)
    for postal_code, new_row_list in new_data.items():
        logger.debug(f"Inspecting {len(new_row_list)} new records from {postal_code}")

        # Pull the current rows from the source
        current_row_list = current_data[postal_code]
        logger.debug(
            f"Comparing against {len(current_row_list)} records from the current database"
        )

        # Loop through the rows in this source
        for new_row in new_row_list:
            # Identify new rows that are identical to a record in the current database
            if not any(
                r for r in current_row_list if r["hash_id"] == new_row["hash_id"]
            ):
                # If not, it's either a new record or an amendment.
                # So it goes in our change list
                changed_dict[postal_code].append(new_row)

        # Log where we stand
        change_list = changed_dict[postal_code]
        logger.debug(
            f"{len(change_list)} changed rows ({round((len(change_list)/len(new_row_list))*100, 2)}%)"
        )

    # Pass it out
    return changed_dict


def regroup_by_source(data_list: typing.List) -> typing.DefaultDict[str, typing.List]:
    """Regroup the provided list by its source field.

    Args:
        data_list: A list of dictionaries presumed to have a "postal_code" field.

    Returns: A dictionary keyed by postal code. Each value is a list of all records with that value.
    """
    regrouped_dict = defaultdict(list)
    for row in data_list:
        regrouped_dict[row["postal_code"]].append(row)
    return regrouped_dict


def flatten_grouped_data(grouped_data: typing.Dict[str, typing.List]):
    """Flatten a dictionary of data grouped by source down to a single list.

    Args:
        grouped_data (dict): The grouped data

    Returns a list of the data for all sources
    """
    return list(chain(*grouped_data.values()))


if __name__ == "__main__":
    run()



import datetime

from ..schema import BaseTransformer

# import typing


class Transformer(BaseTransformer):
    """Transform Hawaii raw data for consolidation."""

    postal_code = "HI"
    # Company,Date,PDF url,location,jobs

    fields = dict(
        company="Company",
        notice_date="Date",
        location="location",
        jobs="jobs",
    )

    date_format = "%Y-%m-%d"

    date_corrections = {
        "* Hawaiian Airlines Amended September 16, 2020": datetime.datetime(
            2020, 9, 16
        ),
        "*Hyatt Regency Waikiki Update December 14, 2020": datetime.datetime(
            2020, 12, 14
        ),
        "*Grand Hyatt Kauai Resort & Spa Amendment #2 August 13, 2021": datetime.datetime(
            2021, 8, 13
        ),
        "*American Machinery Update October 21, 2020": datetime.datetime(2020, 10, 21),
        "** Correction to FOH Hospitality Inc.": None,
        "*Hawaiian Airlines Amendment": None,
        "* Correction to Marriott Resort Hospitality Corporation": None,
        "* DFS Update October 30, 2020": datetime.datetime(2020, 10, 30),
        "*Alohilani Resort Amendment": None,
        "*** Correction to HV Global Management Corporation": None,
        "* Princeville Resort updated March 24, 2020": datetime.datetime(2020, 3, 24),
        "*Grand Hyatt Kauai Resort & Spa Amendment October 30, 2020": datetime.datetime(
            2020, 10, 30
        ),
        "* Waikoloa Beach Marriott Resort & Spa Amended June 9, 2020": datetime.datetime(
            2020, 6, 9
        ),
        "*JTB Hawaii, Inc. Supplement October 30, 2020": datetime.datetime(
            2020, 10, 30
        ),
        "*Hawaiian Airlines Second Amended October 14, 2020": datetime.datetime(
            2020, 10, 14
        ),
        "*Flying Food Group, LLC Amended 10/12/2021": datetime.datetime(2021, 10, 12),
        "*Errata to Amended WARN": None,
        "September 10. 2021 He-Man Landscaping, LLC": datetime.datetime(2021, 9, 21),
        "September 10. 2021": datetime.datetime(2021, 9, 21),
    }

    jobs_corrections = {
        "": None,
    }

    def transform_company(self, value: str) -> str:
        """Transform a raw company name.

        Args:
            value (str): The raw company string provided by the source

        Returns: A string object ready for consolidation.
        """
        return value.split("\n")[0].strip()



import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Michigan raw data for consolidation."""

    postal_code = "MI"
    fields = dict(
        company="Company Name",
        location="City",
        notice_date="Date Received",
        jobs="Number of Layoffs",
    )
    date_format = "%m/%d/%Y"
    jobs_corrections = {
        "80*": 80,
        "Unreported": None,
    }

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "closing" in row["Incident Type"].lower() or None



from pathlib import Path

import pytest

from warn_transformer import consolidate


@pytest.mark.vcr()
def test_consolidate():
    """Test consolidate."""
    this_dir = Path(__file__).parent
    input_dir = this_dir / "data" / "raw"
    consolidate.run(input_dir)



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Alabama raw data for consolidation."""

    postal_code = "AL"
    fields = dict(
        company="Company",
        location="City",
        notice_date="Initial Report Date",
        effective_date="Planned Starting Date",
        jobs="Planned # Affected Employees",
    )
    date_format = "%m/%d/%Y"
    date_corrections = {"01/01/0001": datetime(2020, 1, 1)}

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "closing" in row["Closing or Layoff"].lower() or None



import csv
import logging
import typing
from importlib import import_module
from pathlib import Path

from . import utils

logger = logging.getLogger(__name__)


def run(
    input_dir: Path = utils.WARN_TRANSFORMER_OUTPUT_DIR / "raw",
    source: typing.Optional[str] = None,
) -> Path:
    """Consolidate raw data using a common data schema.

    Args:
        input_dir (Path): The directory where our raw data files are stored.
        source (string): The slug of a source you'd like to transform as a one-off (optional)

    Returns: The path to our consolidated comma-delimited file.
    """
    logging.basicConfig(level="DEBUG", format="%(asctime)s - %(name)s - %(message)s")

    # Get all of the transformers
    transformer_list = utils.get_all_transformers()
    logger.info(f"Consolidating {len(transformer_list)} sources")

    # If a source is provided, limit the list
    if source:
        transformer_list = [t for t in transformer_list if source.lower() in t.lower()]

    # Loop through them
    obj_list = []
    for t in transformer_list:
        # Get the module
        module = import_module(f"warn_transformer.transformers.{t}")

        # Transform the data
        source_list = module.Transformer(input_dir).transform()
        if len(source_list) <= 3:
            logger.warning(
                f"{t.upper()} data quality problem: {len(source_list):,} items found."
            )
        else:
            logger.debug(f"{t.upper()} data {len(source_list):,} items found.")

        # Add it to the master list
        obj_list += source_list

    # Drop duplicates by using the hash as a unique identifer
    unique_list = list({d["hash_id"]: d for d in obj_list}.values())
    logger.debug(f"Dropped {len(obj_list) - len(unique_list)} duplicates")

    # Get the output directory
    processed_dir = utils.WARN_TRANSFORMER_OUTPUT_DIR / "processed"
    if not processed_dir.exists():
        processed_dir.mkdir(parents=True)

    # Output a consolidated CSV
    consolidated_path = processed_dir / "consolidated.csv"
    logger.debug(f"Writing {len(unique_list)} records to {consolidated_path}")
    with open(consolidated_path, "w") as fh:
        writer = csv.DictWriter(fh, unique_list[0].keys())
        writer.writeheader()
        writer.writerows(unique_list)

    # Return the path
    return consolidated_path


if __name__ == "__main__":
    run()



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Indiana raw data for consolidation."""

    postal_code = "IN"
    fields = dict(
        company="Company",
        location="City",
        notice_date="Notice Date",
        effective_date="LO/CL Date",
        jobs="Affected Workers",
    )
    date_format = ["%m/%d/%Y", "%m/%d/%y", "%B %Y", "%Y", "%b %Y", "%m/%Y"]
    jobs_corrections = {
        "97 (in MI)0 (in IN)": 0,
        "100+": 100,
        "62 MAY be affected": 62,
        "5 in Indiana": 5,
        "Unknown": None,
        "75 in Indiana": 75,
        "40-50": 40,
        "100-130": 100,
        "4 Hoosiers": 4,
        "Undisclosed at this time": None,
        "500 Nationwide": None,
        "NA": None,
        "103 (REVISED) 10/22/2020 108": 103,
        "Entire Plant": None,
        "All": None,
    }
    date_corrections = {
        "01/30/1202": datetime(2012, 1, 30),
        "April/June 2020": datetime(2020, 4, 1),
        "Unknown": None,
        "Q1 2019": datetime(2019, 1, 1),
        "Q1 2018": datetime(2018, 1, 1),
        "Sept. 2016": datetime(2016, 9, 1),
        "No closure date announced. Layoffs to commence 05/27/2015": datetime(
            2015, 5, 27
        ),
        "TBD": None,
        "09/22/2014-12/07/2014": datetime(2014, 9, 22),
        "08/18/2014-12/31/2014": datetime(2014, 8, 18),
        "End of 2013": datetime(2013, 12, 31),
        "Mid-Year 2014": datetime(2014, 6, 15),
        "02/29/2013": datetime(2013, 2, 28),
        "year end 2014": datetime(2014, 12, 31),
        "4th Qtr 2012": datetime(2012, 9, 1),
        "Mid February 2012": datetime(2012, 2, 14),
        "3rd Qtr 2012": datetime(2012, 6, 1),
        "LO-01/14/2011 CL-End of 2012": datetime(2011, 1, 14),
        "Prior to the end of 2009 (as stated in the WARN notice)": datetime(
            2009, 12, 31
        ),
        "No closure date announced. Layoffs": None,
        "1st Quarter 2009": datetime(2009, 1, 1),
        "02/02/2009\xa0to\xa0\xa012/30/2009": datetime(2009, 2, 2),
        "3rd Quarter of 2009": datetime(2009, 6, 1),
        "August to December 2008": datetime(2008, 8, 1),
        "10/37/2008": datetime(2008, 10, 27),
        "2/29/2013": datetime(2013, 2, 28),
        "LO-1/14/2011 CL-End of 2012": datetime(2011, 1, 14),
        "3rd quarter of 2009": datetime(2009, 6, 1),
    }

    def prep_row_list(
        self, row_list: typing.List[typing.Dict]
    ) -> typing.List[typing.Dict]:
        """Make necessary transformations to the raw row list prior to transformation.

        Args:
            row_list (list): A list of raw rows of data from the source.

        Returns: The row list minus empty records
        """
        # Do the standard stuff
        row_list = super().prep_row_list(row_list)

        # Cut rows with data-free revisions
        return [r for r in row_list if r["Affected Workers"] != "N/A"]

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        # Try corrections before we edit the string
        try:
            dt = self.date_corrections[value]
            if dt:
                return str(dt.date())
            else:
                assert dt is None
                return dt
        except KeyError:
            pass

        # A little custom clean up based on the weird stuff from this source
        value = value.replace("starting", "")
        value = value.strip().split(" and ")[0].strip()
        value = value.strip().split(" to ")[0].strip()
        value = value.strip().split(" through ")[0].strip()
        value = value.strip().split(" - ")[0].strip()
        value = value.strip().split(" & ")[0].strip()
        value = value.strip().split("\xa0to ")[0].strip()
        value = value.strip().split(" – ")[0].strip()
        value = value.strip().split("-")[0].strip()

        # The same old stuff
        return super().transform_date(value)

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        whitelist = ["CL", "CL -Relocating", "LO and CL", "LO/CL", "PENDING CL"]
        return row["Notice Type"] in whitelist or None



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Nebraska raw data for consolidation."""

    postal_code = "NE"
    fields = dict(
        company="Company",
        location="City",
        notice_date="Date",
        jobs="Jobs Affected",
    )
    date_format = "%m/%d/%Y"
    date_corrections = {
        "12/19/2022\xa0\xa0\n\xa0 11/2/2022": datetime(2022, 11, 2),
        "12/19/2022\xa0\xa0\n\xa0 11/02/2022": datetime(2022, 11, 2),
    }
    jobs_corrections = {
        "100+": 100,
        "5-9": 5,
        "3-5": 3,
        "a few": 1,
    }

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        return "closure" in row["Type"].lower() or None



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform New York raw data for consolidation."""

    postal_code = "NY"
    fields = dict(
        company=lambda row: row["company_name"] or row["Company"] or None,
        location="City",
        notice_date=lambda row: row["notice_dated"] or row["Notice Date"] or None,
        effective_date="Layoff Date",
        jobs="Number Affected",
    )
    date_format = ("%Y-%m-%d %H:%M:%S", "%m/%d/%Y", "%Y-%m-%d")
    date_corrections = {
        "929/2022": datetime(2022, 9, 29),
        "3/6/3023": datetime(2023, 3, 6),
        "2": datetime(2021, 2, 12),
        "2/2/2024`": datetime(2024, 2, 2),
    }

    def prep_row_list(
        self, row_list: typing.List[typing.Dict]
    ) -> typing.List[typing.Dict]:
        """Make necessary transformations to the raw row list prior to transformation.

        Args:
            row_list (list): A list of raw rows of data from the source.

        Returns: The row list minus empty records
        """
        # Do the standard stuff
        row_list = super().prep_row_list(row_list)

        # Split records from scrape from those in the archival set
        scraped_list = [r for r in row_list if r["notice_url"]]
        archival_list = [r for r in row_list if not r["notice_url"]]
        assert len(scraped_list) + len(archival_list) == len(row_list)

        # Remove records from the scrape that are covered by the more detailed archival file
        cutoff = datetime(2021, 6, 30)
        keep_list = []
        for r in scraped_list:
            dt_str = self.transform_date(r["notice_dated"])
            assert isinstance(dt_str, str)
            dt = datetime.strptime(dt_str, "%Y-%m-%d")
            if dt > cutoff:
                keep_list.append(r)

        # Add them back together
        prepped_list = keep_list + archival_list

        # Return it
        return prepped_list

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        if not value:
            return None
        value = value.split()[0].replace(",", "").replace(";", "")
        return super().transform_date(value)

    def check_if_temporary(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a temporary or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        value = row["Dislocation Type"].lower()
        if "possible" in value or "potential" in value:
            return None
        return "temp" in value or None

    def check_if_closure(self, row: typing.Dict) -> typing.Optional[bool]:
        """Determine whether a row is a closure or not.

        Args:
            row (dict): The raw row of data.

        Returns: A boolean or null
        """
        value = row["Dislocation Type"].lower()
        if "possible" in value or "potential" in value or "temp" in value:
            return None
        return "clos" in value or None



import logging
import os
import typing
from pathlib import Path

from bln.client import Client

from . import utils

BLN_API_KEY = os.getenv("BLN_API_TOKEN")
BLN_PROJECT_ID = "UHJvamVjdDpiZGM5NmU1MS1kMzBhLTRlYTctODY4Yi04ZGI4N2RjMzQ1ODI="

logger = logging.getLogger(__name__)


def run(
    download_dir: Path = utils.WARN_TRANSFORMER_OUTPUT_DIR / "raw",
    source: typing.Optional[str] = None,
):
    """Download all the CSVs in the WARN Notice project on biglocalnews.org.

    Args:
        download_dir (Path): The directory where files will be downloaded.
        source (str): The postal code of the source to download. Default is all sources.
    """
    logging.basicConfig(level="DEBUG", format="%(asctime)s - %(name)s - %(message)s")

    # Login to BLN.
    c = Client(BLN_API_KEY)

    # Get the Warn Act Notices project.
    p = c.get_project_by_name("WARN Act Notices")

    # Get all the files in the project.
    file_list = [f["name"] for f in p["files"]]

    # If a source is provided, limit the list
    if source:
        file_list = [f for f in file_list if source.lower() in f.lower()]
        if len(file_list) == 0:
            logger.debug(f"No source '{source}' found")

    # Make the download directory, if it doesn't already exist.
    if not download_dir.exists():
        download_dir.mkdir(parents=True)

    # Download all the files.
    for i, f in enumerate(sorted(file_list)):
        logger.debug(
            f"Download {f} to {download_dir} as file {i+1:02d} of {len(file_list):02d}"
        )
        c.download_file(BLN_PROJECT_ID, f, output_dir=download_dir)


if __name__ == "__main__":
    run()



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform New Mexico raw data for consolidation."""

    postal_code = "NM"
    fields = dict(
        company="JOB SITE NAME",
        location="CITY NAME",
        notice_date="NOTICE DATE",
        effective_date="LAYOFF DATE",
        jobs="TOTAL LAYOFF NUMBER",
    )
    date_format = ("%d-%b-%Y", "%d-%b-%y", "%m/%d/%Y", "%m/%d/%y")
    date_corrections = {
        "1/0/00": None,
    }
    jobs_corrections = {
        "Not Disclosed": None,
        "?": None,
        "N/A": None,
    }
